import 'package:flutter/material.dart';
import 'package:flutter_chat_types_advn/flutter_chat_types.dart' as types;
import 'package:flutter_chat_types_advn/flutter_chat_types.dart';
import 'package:flutter_link_previewer_advn/flutter_link_previewer.dart'
    show LinkPreview;

import '../../helper/status_helper.dart';
import '../../models/emoji_enlargement_behavior.dart';
import '../state/inherited_chat_theme.dart';
import '../state/inherited_user.dart';
import 'user_name.dart';
// import '../../../../../../shared/helper/status_helper.dart';

/// A class that represents text message widget with optional link preview.
class CallMessage extends StatelessWidget {
  /// Creates a text message widget from a [types.CallMessage] class.
  const CallMessage({
    key,
    required this.emojiEnlargementBehavior,
    required this.hideBackgroundOnEmojiMessages,
    required this.message,
    this.nameBuilder,
    this.onPreviewDataFetched,
    this.options = const CallMessageOptions(),
    required this.showName,
    required this.usePreviewData,
    required this.userAgent,
    this.callBackCallAgain,
  }): super(key: key);

  final Function(bool? isVideo)? callBackCallAgain;

  /// See [Message.emojiEnlargementBehavior].
  final EmojiEnlargementBehavior emojiEnlargementBehavior;

  /// See [Message.hideBackgroundOnEmojiMessages].
  final bool hideBackgroundOnEmojiMessages;

  /// [types.CallMessage].
  final types.CallMessage message;

  /// This is to allow custom user name builder
  /// By using this we can fetch newest user info based on id
  final Widget Function(String userId)? nameBuilder;

  /// See [LinkPreview.onPreviewDataFetched].
  final void Function(types.CallMessage, types.PreviewData)?
      onPreviewDataFetched;

  /// Customisation options for the [CallMessage].
  final CallMessageOptions options;

  /// Show user name for the received message. Useful for a group chat.
  final bool showName;

  /// Enables link (URL) preview.
  final bool usePreviewData;

  /// User agent to fetch preview data with.
  final String? userAgent;

  @override
  Widget build(BuildContext context) {
    final theme = InheritedChatTheme.of(context).theme;
    final user = InheritedUser.of(context).user;

    return Container(
      margin: EdgeInsets.symmetric(
        horizontal: theme.messageInsetsHorizontal!,
        vertical: theme.messageInsetsVertical!,
      ),
      child: IntrinsicWidth(child: _callWidgetBuilder(user, context)),
    );
  }

  Widget _callWidgetBuilder(
    types.User user,
    BuildContext context,
    // ignore: prefer_expression_function_bodies
  ) {
    return Column(
      crossAxisAlignment: CrossAxisAlignment.start,
      children: [
        if (showName)
          nameBuilder?.call(message.author.id) ??
              UserName(author: message.author),
        Column(
          crossAxisAlignment: CrossAxisAlignment.start,
          children: [
            if (message.statusCall != StatusCall.actionCallEnd) ...[
              Row(
                children: [
                  Image.asset(
                    StatusHelper.getImageTypeCall(
                      status: convertEnumCallString(status: message.statusCall),
                      type: message.callType,
                    ),
                    package: 'chat_call_core',
                    width: 20,
                    height: 20,
                  ),
                  const SizedBox(width: 8),
                  Text(
                    message.text,
                    style: TextStyle(
                      color: StatusHelper.getColorCall(
                        status: convertEnumCallString(
                          status: message.statusCall,
                        ),
                      ),
                    ),
                  ),
                ],
              ),
            ],
            const SizedBox(height: 6),
            if (message.statusCall == StatusCall.actionCallEnd) ...[
              if (message.timerCall?.isNotEmpty == true) ...[
                Column(
                  crossAxisAlignment: CrossAxisAlignment.start,
                  children: [
                    Text(
                      message.text,
                      style: TextStyle(
                        color: StatusHelper.getColorCall(
                          status: convertEnumCallString(
                            status: message.statusCall,
                          ),
                        ),
                      ),
                    ),
                    const SizedBox(height: 4),
                    Row(
                      children: [
                        Image.asset(
                          StatusHelper.getImageTypeCall(
                            status: convertEnumCallString(
                              status: message.statusCall,
                            ),
                            type: message.callType,
                          ),
                          package: 'chat_call_core',
                          width: 20,
                          height: 20,
                        ),
                        const SizedBox(width: 8),
                        Text(
                          timerCallConvert(message.timerCall),
                          style: TextStyle(
                            color: StatusHelper.getColorCall(
                              status: convertEnumCallString(
                                status: message.statusCall,
                              ),
                            ),
                          ),
                        ),
                      ],
                    ),
                  ],
                ),
              ] else ...[
                Row(
                  children: [
                    Image.asset(
                      StatusHelper.getImageTypeCall(
                        status: convertEnumCallString(
                          status: message.statusCall,
                        ),
                        type: message.callType,
                      ),
                      package: 'chat_call_core',
                      width: 20,
                      height: 20,
                    ),
                    const SizedBox(width: 8),
                    Text(
                      message.text,
                      style: TextStyle(
                        color: StatusHelper.getColorCall(
                          status: convertEnumCallString(
                            status: message.statusCall,
                          ),
                        ),
                      ),
                    ),
                  ],
                ),
              ],
            ],
            const SizedBox(height: 4),
            Container(
              height: 1,
              width: MediaQuery.of(context).size.width / 3,
              color: Colors.grey.shade200,
            ),
            const SizedBox(height: 4),
            GestureDetector(
              onTap: () {
                if (callBackCallAgain != null) {
                  callBackCallAgain!(message.isVideo);
                }
              },
              child: const Align(
                alignment: Alignment.bottomCenter,
                child: Text(
                  'Gọi lại',
                  style: TextStyle(
                    color: Colors.black54,
                  ),
                ),
              ),
            ),
          ],
        ),
      ],
    );
  }

  // ignore: member-ordering
  String timerCallConvert(String? timerCall) {
    if (timerCall?.isNotEmpty == true) {
      return timerCall ?? '0 Phút 0 Giây';
    }
    return '0 Phút 0 Giây';
  }
}

@immutable
class CallMessageOptions {
  const CallMessageOptions({
    this.onLinkPressed,
    this.openOnPreviewImageTap = false,
    this.openOnPreviewTitleTap = false,
  });

  /// Custom link press handler.
  final void Function(String)? onLinkPressed;

  /// See [LinkPreview.openOnPreviewImageTap].
  final bool openOnPreviewImageTap;

  /// See [LinkPreview.openOnPreviewTitleTap].
  final bool openOnPreviewTitleTap;
}
